// --------------------------------------------------------------------
// Author  : mashimonator
// Create  : 2009/11/12
// Update  : 2009/11/12
// Description : フォームのパスワード入力支援
// --------------------------------------------------------------------

/*@cc_on 
var doc = document;
eval('var document = doc');
@*/
var passwordTextExtension = {
	//-----------------------------------------
	// 設定値
	//-----------------------------------------
	conf : {
		targetClass : 'pwdExt', // 対象とするクラス
		font : 'normal 1em inherit', // フォントのスタイル
		color : '#fff', // 文字色
		bgColor : '#000', // 背景色
		border : 'none', // 枠線
		speed : 50, // 文字を隠す速さ
		prefix : '_target'
	},
	storage : {
		target : null,
		view : null,
		checkTimer : null,
		hideTimer : null,
		shim : null
	},
	//-----------------------------------------
	// 初期処理
	//-----------------------------------------
	init : function() {
		var ipts = passwordTextExtension.getTargetElements( 'input', passwordTextExtension.conf.targetClass );
		for ( var i = 0, len = ipts.length; i < len; i++ ) {
			if ( ipts[i].type.toLowerCase() == 'password' ) {
				// 対応する表示エリアを生成
				passwordTextExtension.createViewArea(ipts[i]);
				// イベントに関数を付加
				passwordTextExtension.addEvent( ipts[i], 'focus', passwordTextExtension.focus );
				passwordTextExtension.addEvent( ipts[i], 'blur', passwordTextExtension.blur );
			}
		}
		// shimを生成
		passwordTextExtension.storage.shim = passwordTextExtension.createShim();
	},
	//-----------------------------------------
	// パスワード表示要素を生成
	//-----------------------------------------
	createViewArea : function( element ) {
		// 親要素取得
		var parent = element.parentNode;
		// div要素を生成
		var div = document.createElement('div');
		// スタイルをセット
		div.id = element.id + passwordTextExtension.conf.prefix;
		div.style.margin = '0';
		div.style.padding = '3px 5px 3px 5px';
		div.style.border = passwordTextExtension.conf.border;
		div.style.color = passwordTextExtension.conf.color;
		div.style.backgroundColor = passwordTextExtension.conf.bgColor;
		div.style.font = passwordTextExtension.conf.font;
		div.style.zIndex = '200';
		div.style.textAlign = 'center';
		div.style.verticalAlign = 'middle';
		div.style.position = 'absolute';
		div.innerHTML = element.value;
		passwordTextExtension.setOpacity( div, 0 );
		div.style.visibility = 'hidden';
		// 要素を追加
		parent.appendChild(div);
	},
	//-----------------------------------------
	// shimを生成する
	//-----------------------------------------
	createShim : function() {
		// iframe要素を生成
		var iframe = document.createElement('iframe');
		// スタイルをセット
		iframe.setAttribute('frameBorder', '0');
		iframe.setAttribute('scrolling', 'no');
		iframe.style.border = 'none';
		iframe.style.zIndex = '150';
		passwordTextExtension.setSize( iframe, 1, 1 );
		iframe.style.visibility = 'hidden';
		// 要素を追加
		document.body.appendChild(iframe);
		return iframe;
	},
	//-----------------------------------------
	// shimのスタイルをリセットする
	//-----------------------------------------
	resetShim : function() {
		passwordTextExtension.setSize( passwordTextExtension.storage.shim, 1, 1 );
		passwordTextExtension.setPosition( passwordTextExtension.storage.shim, 0, 0 );
		passwordTextExtension.storage.shim.style.visibility = 'hidden';
	},
	//-----------------------------------------
	// focusイベントでの処理
	//-----------------------------------------
	focus : function() {
		// 入力中のinput要素を取得
		passwordTextExtension.storage.target = document.getElementById(this.id);
		// 対応するパスワード表示エリアを取得
		passwordTextExtension.storage.view = document.getElementById(this.id + passwordTextExtension.conf.prefix);
		// ポジション取得
		var pos = passwordTextExtension.getElementPosition(passwordTextExtension.storage.target);
		// サイズ取得
		var size = passwordTextExtension.getElementSize(passwordTextExtension.storage.target);
		// パスワード表示エリアの位置をセット
		passwordTextExtension.setPosition(passwordTextExtension.storage.view, pos.top, pos.left+size.width+3);
		// shimの位置をセット
		passwordTextExtension.setPosition(passwordTextExtension.storage.shim, pos.top, pos.left+size.width+3);
		// Timerスタート
		passwordTextExtension.storage.checkTimer = window.setTimeout('passwordTextExtension.checkPassword(\'\')', 200);
	},
	//-----------------------------------------
	// blurイベントでの処理
	//-----------------------------------------
	blur : function() {
		// Timerストップ
		window.clearTimeout(passwordTextExtension.storage.checkTimer);
		passwordTextExtension.storage.checkTimer = null;
		// 入力中のinput要素をクリア
		passwordTextExtension.storage.target = null;
		// パスワード表示エリアを非表示
		passwordTextExtension.setOpacity( passwordTextExtension.storage.view, 0 );
		passwordTextExtension.storage.view = null;
		// shimをリセット
		passwordTextExtension.resetShim();
	},
	//-----------------------------------------
	// 入力値のチェック処理
	//-----------------------------------------
	checkPassword : function( oldStr ) {
		var newStr = passwordTextExtension.storage.target.value;
		if ( newStr != '' && newStr != oldStr ) {
			// 入力値をセット
			passwordTextExtension.storage.view.innerHTML = newStr;
			// タイマークリア
			window.clearTimeout(passwordTextExtension.storage.hideTimer);
			passwordTextExtension.storage.hideTimer = null;
			// shimのサイズ設定
			var size = passwordTextExtension.getElementSize(passwordTextExtension.storage.view);
			passwordTextExtension.setSize( passwordTextExtension.storage.shim, size.width, size.height );
			// shimを表示
			passwordTextExtension.storage.shim.style.visibility = 'visible';
			// パス表示エリアを表示
			passwordTextExtension.storage.view.style.visibility = 'visible';
			passwordTextExtension.setOpacity( passwordTextExtension.storage.view, 10 );
			// 非表示処理開始
			passwordTextExtension.hide( passwordTextExtension.storage.view.id );
		}
		passwordTextExtension.storage.checkTimer = window.setTimeout('passwordTextExtension.checkPassword(\'' + newStr + '\')', 200);
	},
	//-----------------------------------------
	// 対象要素の非表示処理
	//-----------------------------------------
	hide : function( id ) {
		var div = document.getElementById(id);
		var opacity = passwordTextExtension.getOpacity( div );
		if ( opacity > 0 ) {
			// 不透明度をセット
			passwordTextExtension.setOpacity( div, opacity-1 );
			// タイマーセット
			passwordTextExtension.storage.hideTimer = window.setTimeout('passwordTextExtension.hide(\'' + passwordTextExtension.storage.view.id + '\')', passwordTextExtension.conf.speed);
		} else {
			// パスワード表示エリアを非表示
			div.style.visibility = 'hidden';
			// shimを非表示
			passwordTextExtension.storage.shim.style.visibility = 'hidden';
			// タイマークリア
			window.clearTimeout(passwordTextExtension.storage.hideTimer);
			passwordTextExtension.storage.hideTimer = null;
		}
	},
	//-----------------------------------------
	// 要素の不透明度を取得する
	//-----------------------------------------
	getOpacity : function( element ) {
		if ( element.alpha ) {
				return element.alpha / 10;
		} else if ( element.style ) {
			if ( element.style.opacity ) {
				return element.style.opacity * 10;
			}
		} 
		return 0;
	},
	//-----------------------------------------
	// 要素の不透明度を設定する
	//-----------------------------------------
	setOpacity : function( element, val ) {
		element.style.opacity = val / 10;
		element.style.filter = 'alpha(opacity=' + val * 10 + ')';
		element.alpha = val * 10;
	},
	//-----------------------------------------
	// 要素のサイズを設定する
	//-----------------------------------------
	setSize : function( element, width, height ) {
		element.style.width = width + 'px';
		element.style.height = height + 'px';
	},
	//-----------------------------------------
	// 要素の位置を設定する
	//-----------------------------------------
	setPosition : function( element, top, left ) {
		element.style.position = 'absolute';
		element.style.top = top + 'px';
		element.style.left = left + 'px';
	},
	//-----------------------------------------
	// 対象要素のポジションを取得する
	//-----------------------------------------
	getElementPosition : function( element ) {
		var offsetTrail = (typeof element == 'string') ? document.getElementById(element): element;
		var x = 0;
		var y = 0;
		while (offsetTrail) {
			x += offsetTrail.offsetLeft;
			y += offsetTrail.offsetTop;
			offsetTrail = offsetTrail.offsetParent;
		}
		if ( navigator.userAgent.indexOf('Mac') != -1 && typeof doc.body.leftMargin != 'undefined' ) {
			x += document.body.leftMargin;
			y += document.body.topMargin;
		}
		return ({ 'left': x, 'top': y });
	},
	//-----------------------------------------
	// 対象要素のサイズを取得する
	//-----------------------------------------
	getElementSize : function( element ) {
		var w = element.offsetWidth;
		var h = element.offsetHeight;
		return ({ 'width': w, 'height': h });
	},
	//-----------------------------------------
	// ターゲットタグを取得する
	//-----------------------------------------
	getTargetElements : function( tag, cls ) {
		var elements = new Array();
		var targetElements = document.getElementsByTagName(tag.toUpperCase());
		for ( var i = 0, len = targetElements.length; i < len; i++ ) {
			if ( targetElements[i].className.match(cls) ) {
				elements[elements.length] = targetElements[i];
			}
		}
		return elements;
	},
	//-----------------------------------------
	// イベントに関数を付加する
	//-----------------------------------------
	addEvent : function( target, event, func ) {
		try {
			target.addEventListener(event, func, false);
		} catch (e) {
			target.attachEvent('on' + event, (function(el){return function(){func.call(el);};})(target));
		}
	}
}
// 実行
passwordTextExtension.addEvent( window, 'load', passwordTextExtension.init );